/*
 * Donnees.h : 
 * Copyright (C) 2007 Florence Zara, LIRIS
 *               florence.zara@liris.univ-lyon1.fr
 *               http://liris.cnrs.fr/florence.zara/
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/** \file Donnees.h
* \brief Strutures de donnees generales
*/ 


#ifndef _DONNEES_H
#define _DONNEES_H

#include <vector>



/**
*\brief Vecteur de 3 coordonnees.
 */
class Vect3D
{
	
protected:
	
	/// Premiere coordonnee
	float x1;
	
	/// Seconde coordonnee
	float x2;
	
	/// Troisieme coordonnee
	float x3;
	
public:	
		
		/*! Constructeur vide */
		Vect3D() : x1(0.0), x2(0.0), x3(0.0) {}
	
	/*! Ajout d un constructeur */
	inline Vect3D(float x, float y, float z) 
    {
		x1 = x; 
		x2 = y; 
		x3 = z;
    }
	
	/*! Ajout d un constructeur de copies physiques */
	inline Vect3D(const Vect3D &vect3d)
    {
		x1 = vect3d.x1;
		x2 = vect3d.x2;
		x3 = vect3d.x3;
    }
	
	/*! Surcharge de = */
	inline Vect3D &operator=(const Vect3D &vect3d) 
	{
		x1 = vect3d.x1;
		x2 = vect3d.x2;
		x3 = vect3d.x3;
		
		return *this; 
	}
	
	/*! Ajout d un destructeur */
	~Vect3D() {};
	
};


/** 
* \brief Coordonnees 3D.
*/
class Coord : Vect3D
{    
	
public:      
	
	/*! Constructeur vide */
	Coord() : Vect3D() {};
	
	/*! Ajout d un constructeur */
	Coord(float x, float y, float z) : Vect3D(x, y, z) {};
	
	/*! Acces en lecture a la premiere coordonnee */	
	inline float getX() {return x1;}	
	
	/*! Acces en lecture a la seconde coordonnee */	
	inline float getY() {return x2;}	
	
	/*! Acces en lecture a la troisieme coordonnee */	
	inline float getZ() {return x3;}	
	
	/*! Acces en ecriture a la premiere coordonnee */	
	inline void setX(float x) {x1 = x;}	
	
	/*! Acces en ecriture a la seconde coordonnee */	
	inline void setY(float y) {x2 = y;}	
	
	/*! Acces en ecriture a la troisieme coordonnee */	
	inline void setZ(float z) {x3 = z;}	
	
	/*! Surcharge de l operateur = */
	Coord& operator=(Coord b)
	{
		setX(b.getX());
		setY(b.getY());
		setZ(b.getZ());
		
		return *this;
	}
	
	/*! Surcharge de += */
	inline Coord& operator+=(Coord b) 
	{
		float x, y, z;
		
		x = getX() + b.getX(); 
		y = getY() + b.getY(); 
		z = getZ() + b.getZ(); 
		
		setX(x);
		setY(y);
		setZ(z);
		
		return *this;
		
	}	
	
	/*! Surcharge de -= */
	Coord& operator-=(Coord b) 
	{
		float x, y, z;
		
		x = getX() - b.getX(); 
		y = getY() - b.getY(); 
		z = getZ() - b.getZ(); 
		
		setX(x);
		setY(y);
		setZ(z);
		
		return *this;
	}
	
	/*! Surcharge de - */
	Coord operator-(const Coord& b) const
    {
		Coord tmp(*this);
		
		return tmp -= b;
    }
	
	/*! Surchage de l operateur * */
	Coord operator *(float alpha)
	{
		return Coord((getX() * alpha),
					 (getY() * alpha),
					 (getZ() * alpha));
	}
	
	/*! Calcul du min (Coord, Coord) */
	friend Coord min(Coord a, Coord b) 
    {
		return Coord( 
					  std::min(a.getX(), b.getX()), 
					  std::min(a.getY(), b.getY()), 
					  std::min(a.getZ(), b.getZ())
					  );
    }
	
	/*! Calcul du max (Coord, Coord) */
	friend Coord max(Coord a, Coord b) 
    {
		return Coord( 
					  std::max(a.getX(), b.getX()), 
					  std::max(a.getY(), b.getY()), 
					  std::max(a.getZ(), b.getZ()));
    }
	
	/*! Methode amie pour la surcharge de l operateur + */
	friend Coord operator+(Coord a, Coord b);
	
	/*! Methode amie pour la surcharge de l operateur - */
	friend Coord operator-(Coord a, Coord b);
	
	/*! Ajout d un destructeur */
	~Coord() {}
	
};


/**
* Methode amie de la class Coord pour la surcharge de l operateur +.
 */
Coord operator+(Coord a, Coord b);


/**
* Methode amie de la class Coord pour la surcharge de l operateur -.
 */
Coord operator-(Coord a, Coord b);


/** 
* \brief Caracteristiques d une couleur : composants RGB et alpha.
*/
class Couleur : Vect3D
{
public:
	
	/// Composante alpha de la couleur
	float _alpha;
	
	
	/*! Constructeur vide */
	Couleur() : Vect3D() {_alpha = 0.0;};
	
	/*! Ajout d un constructeur */
	inline Couleur(float r, float g, float b) : Vect3D(r, g, b) {_alpha = 0.0;};
	
	/*! Ajout d un constructeur */
	inline Couleur(float r, float g, float b, float a) : Vect3D(r, g, b) {_alpha = a;};
	
	/*! Acces en lecture a la premiere coordonnee de la couleur */	
	inline float getR() {return x1;}	
	
	/*! Acces en lecture a la seconde coordonnee de la couleur */	
	inline float getG() {return x2;}	
	
	/*! Acces en lecture a la troisieme coordonnee de la couleur */	
	inline float getB() {return x3;}	
	
	/*! Acces en lecture a la quatrieme coordonnee de la couleur */	
	inline float getAlpha() {return _alpha;}	
	
	/*! Acces en ecriture a la premiere coordonnee de la couleur */	
	inline void setR(float r) {x1 = r;}	
	
	/*! Acces en ecriture a la seconde coordonnee de la couleur */	
	inline void setG(float g) {x2 = g;}	
	
	/*! Acces en ecriture a la troisieme coordonnee de la couleur */	
	inline void setB(float b) {x3 = b;}	
	
	/*! Acces en ecriture a la quatrieme coordonnee de la couleur */	
	inline void setAlpha(float a) {_alpha = a;}	
	
	/*! Surcharge de = */
	inline Couleur &operator=(Couleur couleur)
	{
		setR(couleur.getR());
		setG(couleur.getG());
		setB(couleur.getB());
		setAlpha(couleur.getAlpha());
		
		return *this; 
    }
	
	/*! Ajout d un destructeur */
	~Couleur() {}
	
};


/**
* \brief Caracteristiques d une image : hauteur, largeur et donnees.
 */
struct Image 
{
	
	/// Taille en x
	unsigned long sizeX; 
	
	/// Taille en y
	unsigned long sizeY; 
	
	/// Donnees
	char *data;         
	
};


/**
* \brief Texture determinee par valeurs a et b.
 */
struct Texture 
{
	/// Coordonnee a de la texture 
	float a;
	
	/// Coordonnee b de la texture 
	float b;
};


/**
* \brief Caracteristiques de la lumiere : ambiant, diffus, speculaire, alpha et position.
 */
class Light
{
public:
	
	/// Composante ambiante
	Couleur _amb;
	
	/// Composante diffuse
	Couleur _diff;
	
	/// Composante speculaire
	Couleur _spec;
	
	/// Position
	Coord _pos;
	
	/// Alpha
	float _alpha;
	
	
	/*! Constructeur vide */
	Light() : _amb(), _diff(), _spec(), _pos(), _alpha() {}
	
	/*! Ajout d un constructeur de copies physiques */
	inline Light(const Light &light)
    {
		_amb = light._amb;
		_diff = light._diff;
		_spec = light._spec;
		_pos = light._pos;
		_alpha = light._alpha;
    }
	
	/*! Surcharge de = */
	inline Light &operator=(const Light &light)
	{
		_amb = light._amb;
		_diff = light._diff;
		_spec = light._spec;
		_pos = light._pos;
		_alpha = light._alpha;
		
		return *this; 
    }
	
	/*! Ajout d un destructeur */
	~Light() {};
	
};


/**
* \brief Caracteristiques d un materiau : ambiant, emis, diffus, speculaire, shininess.
 */
class Material
{
public:
	
	/// Composante ambiante
	Couleur _amb;
	
	/// Composante d emission
	Couleur _emis;
	
	/// Composante de diffusion
	Couleur _diff;
	
	/// Composante speculaire
	Couleur _spec;
	
	/// Coefficient de specularite
	float _shin;
	
	
	/*! Constructeur vide */
	Material() : _amb(), _emis(), _diff(), _spec(), _shin() {}
	
	/*! Ajout d un constructeur de copies physiques */
	inline Material(const Material &mater)
    {
		_amb = mater._amb;
		_emis = mater._emis;
		_diff = mater._diff;
		_spec = mater._spec;
		_shin = mater._shin;
    }
	
	/*! Surcharge de = */
	inline Material &operator=(const Material &mater)
    {
		_amb = mater._amb;
		_emis = mater._emis;
		_diff = mater._diff;
		_spec = mater._spec;
		_shin = mater._shin;
		
		return *this; 
		
    }
	
	/*! Ajout d un destructeur */
	~Material() {};
	
};


#endif
