/*
 * ObjetSimuleExpl.cpp : définition des objets animés.
 * Copyright (C) 2007 Florence Zara, LIRIS
 *               florence.zara@liris.univ-lyon1.fr
 *               http://liris.cnrs.fr/florence.zara/
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/** \file ObjetSimule.cpp
\brief Methodes specifiques aux objets a simuler selon la methode Euler explicite.
*/

/** Librairies **/
#include <stdio.h>
#include <vector>
#include <string.h>
#include <math.h>
#include <iostream>
#include <fstream>



#include "Donnees.h"
#include "Noeuds.h"
#include "ObjetSimule.h"


#ifdef WIN32
#include <GL/glut.h> 
#endif

/**
* Constructeur de la class ObjetSimuleExpl.
 */
ObjetSimule::ObjetSimule(std::string fich_param) 
: _Nb_Particules(0),
_Nb_Ressorts(0), 
_RessOS(),
_Size(),
P(0), 
V(0), 
A(0), 
M(0),
Force(0),
Maillage(0)
{
	
	/** Recuperation des parametres mis dans le fichier **/
	Param_mesh(fich_param);
	
	
	/* Calcul du facteur d amortissement des ressorts */
	_RessOS.SetFactAmorti();


}
	

/**
* Initialisation generale.
 */
void ObjetSimule::init()
{
	/** Ouverture des fichiers de donnees de l objet a simuler **/
	setOpenFile();
	
	/** Initialisation des tableaux et construction du maillage 
	a partir du fichier de donnees **/
	setInit();
	
}



/**
* Ouverture des fichiers de donnees.
 */
void ObjetSimule::setOpenFile()
{
	/** Ouverture des fichiers de donnees **/
	
	/// Fichier de donnees des points 
	std::ifstream _FichIn_Points(_Fich_Points.c_str());

	/// Fichier de donnees des faceSet 
	std::ifstream _FichIn_FaceSet(_Fich_FaceSet.c_str());
	
	/// Fichier de donnees des masses
	std::ifstream _FichIn_Masses(_Fich_Masses.c_str());

	
	/* Fichier de donnees des positions */
	if (! _FichIn_Points)
		std::cout << "Erreur d ouverture du fichier de donnees des points : " 
			<< _Fich_Points << std::endl;

	/* Fichier de donnees des faceSet */
	else if (! _FichIn_FaceSet)
		std::cout << "Erreur d ouverture du fichier de donnees des faceSet : " 
			<< _Fich_FaceSet << std::endl; 
	
	/* Fichier des masses */
	else if (! _FichIn_Masses)
		std::cout << "Erreur d ouverture du fichier de donnees des masses : " 
			<< _Fich_Masses << std::endl;
	
	else 
    {
		/* Lecture du nombre de particules */
		_FichIn_Points >> _Nb_Particules;
		std::cout << "Nombre de particules " << _Nb_Particules <<std::endl;
		
		/* Allocation du maillage */
		Maillage = new Mesh();
		
    }//else
	
	/* Fermeture des fichiers */
	_FichIn_Points.close();
	_FichIn_FaceSet.close();
	_FichIn_Masses.close();
	
	std::cout << "Ouverture des fichiers de donnees... " << std::endl;
}


/**
* Initialisation des tableaux et construction du maillage 
 a partir des donnees lues dans le fichier.
 */
void ObjetSimule::setInit()
{ 
	/* Variable intermediaire pour la lecture des coordonnees des positions */
	float tmp_x, tmp_y, tmp_z;
	int tmp;
	
	/* Position min de toutes les particules */
	Coord Pmin (0.0,0.0,0.0);
	
	/* Position max de toutes les particules */
	Coord Pmax (0.0,0.0,0.0);
	
	
	/** Lecture de toutes les particules **/		
	/// Fichier de donnees des points 
	std::ifstream _FichIn_Points(_Fich_Points.c_str());
	
	/// Ne considere pas le premier element du fichier de point 
	/// (correspond au nb de particules deja lu)
	int trash;
	_FichIn_Points >> trash;
	
	/// Lecture des positions
	while (!_FichIn_Points.eof())
	{
		_FichIn_Points >> tmp_x;
		_FichIn_Points >> tmp_y;
		_FichIn_Points >> tmp_z;
		
		/// remplissage de P
		P.push_back(Coord(tmp_x, tmp_y, tmp_z));
	}
 	
	
	/** Lecture de toutes les masses **/
	std::ifstream _FichIn_Masses(_Fich_Masses.c_str());

	/// Lecture des masses
	while (!_FichIn_Masses.eof())
	{
		_FichIn_Masses >> tmp;
		
		/// Remplissage de M
		M.push_back(tmp);
								  
	}							  

	
	/** Calculs intermediaires **/
	/* Calcul de Pmin et Pmax */
	for (int i=0; i<_Nb_Particules; ++i)
    {
		Pmin = min(Pmin, P[i]);
		Pmax = max(Pmax, P[i]);
    }
	 
	
	/* Taille du tissu dans chacune des directions x, y, z */
	_Size.setX( fabs(Pmin.getX() - Pmax.getX()) );
	_Size.setY( fabs(Pmin.getY() - Pmax.getY()) );
	_Size.setZ( fabs(Pmin.getZ() - Pmax.getZ()) );
	
			
	/*** Initialisation des tableaux pour chacune des particules ***/
	// bid = numero de bloc de la particule i
	for (int i=0; i<_Nb_Particules; ++i)
    {
				
		/** Vecteur nulle pour initialiser les vitesses, 
			accel, forces des particules **/
		V.push_back(Coord(0.0,0.0,0.0));
		A.push_back(Coord(0.0,0.0,0.0));
		Force.push_back(Coord(0.0,0.0,0.0));
		
		
		/** Ajout de la particule dans le maillage **/
		/* Construction d une particule */
		Particule *Part = new Particule();
		
		/* Attribution de son identificateur */
		Part->SetId(i);
		
		/* Position de la particule */
		Part->SetPosition(P[i]);
		
		/* Ajout de la particule dans le maillage */
		Maillage->AddParticule(Part);
		
		
    }//for
	
	
	/** Message pour la fin de la lecture des positions et masses **/
	std::cout << "Position/mass read ..." << std::endl;
	
	
	/** Constructions de toutes les facettes du maillage **/
	/* Lecture de toutes les facettes */
	std::ifstream _FichIn_FaceSet(_Fich_FaceSet.c_str());
	
	/// Nombre de facets
	int nb_facet = 0;
		
	/* Construction des facettes */
	while (!_FichIn_FaceSet.eof())
    { 
		/// Remplissage de allfacets
		FacetTriangle facet;
		
		_FichIn_FaceSet >> facet.fi;
		_FichIn_FaceSet >> facet.fj;
		_FichIn_FaceSet >> facet.fk;  
		
		/* Construction de la facette fi, fj, fk */
		Maillage->MakeFace( Maillage->GetParticule(facet.fi), 
							Maillage->GetParticule(facet.fj), 
							Maillage->GetParticule(facet.fk), 
							&_RessOS);
		
		// Recopie dans le tableau des indices des sommets
		Maillage->_VIndices.push_back(facet.fi);
		Maillage->_VIndices.push_back(facet.fj);
		Maillage->_VIndices.push_back(facet.fk);

		nb_facet ++; 
		
    }
	
	/* Taille du tableau des indices des sommets */
	Maillage->_NFacets = nb_facet-1;
	Maillage->_VISize = 3 * Maillage->_NFacets;	

	
	/** Fermeture des fichiers de donnees **/
	_FichIn_FaceSet.close();
	_FichIn_Masses.close();
	_FichIn_Points.close();

	
	// Allocation du tableau des normales
	_NSize = 3 * _Nb_Particules;
	_Normals = new float[_NSize];
			
	/** Modification des  des normales **/
	setNormals();
	
	/** Message pour la fin de la creation du maillage **/
	std::cout << "Mesh build ..." << std::endl;
	
}


/**
* Simulation de l objet.
 */
void ObjetSimule::Simulation(Coord gravite, float viscosite, int Tps)
{
	/* Calcul des forces */  
	//std::cout << "Force.... " << std::endl;
    CalculForce();

	/* Calcul des accelerations (avec ajout de la gravite aux forces) */
	//std::cout << "Accel.... " << std::endl;
	CalculAccel(gravite);
		
	/* Calcul des positions et vitesses */
	if (Tps == 0) 
	{
		/* Calcul des vitesses et positions au temps 0 */
		//std::cout << "PosVit Temps 0.... " << std::endl;
		CalculPositionVitesseTps0(viscosite);
	}
	else
	{
		/* Calcul des positions et vitesses au temps t */
		//std::cout << "PosVit.... " << std::endl;
		CalculPositionVitesse(viscosite);
		
	}
	
	// Affichage des positions
//	AffichagePos(Tps);
	
	/** Modification des normales **/
	setNormals();
	
}


/** 
* Calcul de la normale a une face (a, b, c).
*/
void ObjetSimule::NormaleFace(Coord &normale, int a, int b, int c)
{
	// Coordonnees des vecteurs AB et AC
	Coord AB, AC;
	
	
	/* Coordonnee du vecteur AB */
	AB.setX(P[b].getX() - P[a].getX());
	AB.setY(P[b].getY() - P[a].getY());
	AB.setZ(P[b].getZ() - P[a].getZ());
	
	/* Coordonnee du vecteur AC */
	AC.setX(P[c].getX() - P[a].getX());
	AC.setY(P[c].getY() - P[a].getY());
	AC.setZ(P[c].getZ() - P[a].getZ());
	
	/* Coordonnees de la normale */
	// Coordonnee en x de la normale 
	normale.setX( ((AB.getY()) * (AC.getZ())) - ((AB.getZ()) * (AC.getY())) );
	
	// Coordonnee en y de la normale 
	normale.setY( ((AB.getZ()) * (AC.getX())) - ((AB.getX()) * (AC.getZ())) );
	
	// Coordonnee en z de la normale 
	normale.setZ( ((AB.getX()) * (AC.getY())) - ((AB.getY()) * (AC.getX())) );
	
}


/** 
* Modification du tableau des normales (lissage des normales).
*/
void ObjetSimule::setNormals()
{
	// Normale d une face
	Coord normale;
	
	// Norme d une normale
	double Norme;
	
	// Indices des sommets d une face
	int a, b, c;
	
	/* Parcours des normales des sommets du maillage */
	for(unsigned int i=0; i<_NSize; ++i)
    {
		// Initialisation des normales
		_Normals[i] = 0;
    }
	
	/* Parcours des faces du maillage */
	for(unsigned int i=0; i<Maillage->_NFacets; ++i)
    {
		// Sommets a, b, c de la face
		a = Maillage->_VIndices[3 * i];
		b = Maillage->_VIndices[(3 * i) + 1];
		c = Maillage->_VIndices[(3 * i) + 2];
		
		// Calcul de la normale de la face
		NormaleFace(normale, a, b, c); 
		
		// Modification des normales des sommets de la face
		// Normale du sommet a
		_Normals[(3 * a)] += normale.getX();
		_Normals[(3 * a) + 1] += normale.getY();
		_Normals[(3 * a) + 2] += normale.getZ();
		
		// Normale du sommet b
		_Normals[(3 * b)] += normale.getX();
		_Normals[(3 * b) + 1] += normale.getY();
		_Normals[(3 * b) + 2] += normale.getZ();
		
		// Normale du sommet c
		_Normals[(3 * c)] += normale.getX();
		_Normals[(3 * c) + 1] += normale.getY();
		_Normals[(3 * c) + 2] += normale.getZ();
		
    }//for_i
	
	/* Parcours des normales des sommets */
	for(unsigned int i=0; i<_NSize; i+=3)
    {
		// Norme de la normale
		Norme = sqrt((_Normals[i] * _Normals[i]) 
					 + (_Normals[i+1] * _Normals[i+1]) 
					 + (_Normals[i+2] * _Normals[i+2]));
		
		// Normalisation de la normale
		if (Norme !=0)
        {
			_Normals[i] /= Norme;
			_Normals[i+1] /= Norme;
			_Normals[i+2] /= Norme;
        }
    }
}


/**
* Affichage des positions de chaque particule.
 */
void ObjetSimule::AffichagePos(int tps)
{
	/* Affichage des vecteurs par bloc */
	for(int i=0; i<_Nb_Particules; ++i)
	{
		// Affichage du temps
		std::cout << "[T=" << tps;
		
		// Ecriture du numero de la particule
		std::cout << " ; Part=" << i;
		
		std::cout << " ; P="; 
		
		// Affichage des coordonnees de la position
		std::cout.precision(4);
		
		std::cout << P[i].getX() << " " <<  P[i].getY()  
		<<  " " << P[i].getZ()  << std::endl;
		
    }//for_i
}


/**
* \brief Dessin du maillage.
 * Methode invoquee par le graphe de scene.
 */
void ObjetSimule::affiche()  
{
	// std::cout << "------ ObjetSimule::affiche() ----------- " << std::endl;		

	
	/** Sauvegarde de la matrice courante **/
	glPushMatrix();
	
	/* Mise a l echelle des objets la scene */
	glScalef(0.3, 0.3, 0.3);
	
	/* Translation */
	glTranslatef(0, 3, 1);
	
	
	/** Affichage du maillage **/  
	//// A COMPLETER	
	
	/* Restauration de la matrice courante */
	glPopMatrix();
	
}
