// ==========================================================================
// (c) projet SAPPE
// Author : F. Zara
//
// ==========================================================================


/** \file ObjetSimule.h
    \brief Structures de donnes relatives aux objets simules.
*/

#ifndef OBJET_SIMULE_H
#define OBJET_SIMULE_H



/** Librairies de base **/
#include <stdio.h>
#include <vector>
#include <string.h>
#include <fstream>

#include "Donnees.h"
#include "Noeuds.h"
#include "Mesh.h"


/**
 * \brief Structure de donnees de base des objets simules.
 */
class ObjetSimule: public Noeud
{
 public:

  /*! Constructeur */
  ObjetSimule(std::string fich_param);
	
  /*! Lecture des parametres lies au maillage */
  void Param_mesh(std::string fich_param);
	
  /*! Ouverture des fichiers de donnees */
  void setOpenFile();

  /*! Initialisation des tableaux et construction du maillage 
	  a partir des fichiers de donnees */
  void setInit();

  /*! Initialisation de la simulation */
   void init();

  /*! Calcul des forces appliquees sur les particules */
   void CalculForce();

  /*! Calcul des accelerations (avec ajout de la gravite aux forces) */
   void CalculAccel(Coord g); 
   
   /*! Calcul de la position et de la vitesse des particules au temps 0 pour la methode Leap Frog */
   void CalculPositionVitesseTps0(float visco);
   
   /*! Calcul des positions et des vitesses */
   void CalculPositionVitesse(float visco);

  /*! Simulation de l objet */
   void Simulation(Coord gravite, float viscosite, int Tps);
   
   /*! Affichage des positions de chaque particule */
   void AffichagePos(int tps);
   
   /*! Dessin de l objet */
   void affiche();
   
   /*! Modification du tableau des normales  */
   void setNormals();
   
   /*! Calcul de la normale a une face (a, b, c) */
   void NormaleFace(Coord &normale, int a, int b, int c);   
   
   /*! Modification de la taille du tableau des normales */
   inline void setNSize(int size) {_NSize = size;}

  /*! Acces au maillage de l objet */
  inline Mesh* GetMesh() {return Maillage;}

  /*! Acces au nombre de particules */
  inline int GetNbPart() {return _Nb_Particules;}
  
  /*! Pour acceder aux normales des sommets */
  inline const float *normals() const {return _Normals;}
  
  /*! Pour acceder a la taille du tableau des normales */
  inline const unsigned int nsize() const {return _NSize;}
  
  /*! Acces a la taille du tissu */
  inline Coord GetTissuSize() {return _Size;}

  /*! Acces au nom du fichier des facettes */
  inline std::string GetFacette() {return _Fich_FaceSet;}

  /*! Destructeur de la class ObjetSimule */
  inline virtual ~ObjetSimule();
  
	  
	  
 protected:
  
  /** Donnees relatives aux fichiers de donnees **/
  /// Fichier de donnees contenant les points 
  std::string _Fich_Points;

  /// Fichier de donnees contenant les faceSet 
  std::string _Fich_FaceSet;

  /// Fichier de donnees contenant les masses
  std::string _Fich_Masses;

  
  /** Donnees relatives a la simulation **/
  /// Intervalle de temps 
  float _delta_t;   

  
  /** Donnees relatives au maillage **/
  /// Nombre de particules 
  int _Nb_Particules;

  /// Nombre de ressorts
  int _Nb_Ressorts;

  /// Caracteristiques des ressorts 
  Spring _RessOS;

  /// Longueur du tissu dans chacune des directions x,y,z
  Coord _Size;

 
  /** Structures de donnees pour les calculs **/
  /// Declaration pour les tableaux des particules pour les positions 
  std::vector<Coord> P;
  
  /// Declaration pour les tableaux des particules pour les vitesses 
  std::vector<Coord> V;
  
  /// Declaration pour les tableaux des particules pour les accelerations 
  std::vector<Coord> A;

  /// Declaration du tableau des masses des particules
  std::vector<float> M;

  /// Declaration du tableau des forces des particules 
  std::vector<Coord> Force;
  
  /// Declaration du maillage 
  Mesh *Maillage;
  
  /// Tableau des normales des sommets
  float *_Normals;
  
  /// Taille du tableau des normales
  unsigned int _NSize;
  
 
};


/**
 * Destructeur de la classe ObjetSimule.
 */
ObjetSimule::~ObjetSimule()
{
 	// Liberation du tableau des normales
	if (0 != _Normals)
	{
		delete [] _Normals;
		_Normals = 0;
	}
	
}

#endif
